// Write fragment code here... OK here goes...
// A simple single pass fluid solver by eiffie - trys to do a couple iterations of NS (kinda)

//works on any size buffer so I'm showing smallest bits
#buffer RGBA8
//comment out the buffer shader to see the vectors as colors
#buffershader "smokedBrotPass2.frag"

#vertex

uniform vec2 pixelSize;
varying vec2 coord;

void main(void)
{
	gl_Position =  gl_Vertex;
	coord = (gl_ProjectionMatrix*gl_Vertex).xy;
}

#endvertex

varying vec2 coord;
uniform vec2 pixelSize;

vec3 color(vec2 z) ;

void main() {
	gl_FragColor = vec4(color(coord),1.0);
}

uniform int backbufferCounter;
uniform sampler2D backbuffer;
uniform float time;

#define SubframeMax 0
//#define IterationsBetweenRedraws 1

#TexParameter backbuffer GL_TEXTURE_MIN_FILTER GL_LINEAR
#TexParameter backbuffer GL_TEXTURE_MAG_FILTER GL_LINEAR
#TexParameter backbuffer GL_TEXTURE_WRAP_S GL_REPEAT
#TexParameter backbuffer GL_TEXTURE_WRAP_T GL_REPEAT

float tim=time*0.5;
vec2 forcePt=vec2(0.5+sin(tim*0.333)*0.25,0.0),forceVel=vec2(sin(tim*15.0)*0.01,0.25);

#group TouchAndExplode
uniform float forceMult; slider[2.0,3.0,4.0]
uniform float fade; slider[0.99,0.995,1.0]
uniform float smoulder; slider[1.0,1.044,1.1]
uniform float projMin; slider[0.0,0.24,0.25]
uniform float proj; slider[0.25,0.26,0.5]
uniform float projMax; slider[0.25,0.72,1.0]
//const float forceMult=3.03,fade=0.995,bounce=1.044;
//const float proj=0.26,projMin=0.24,projMax=0.72;//just magic nums - ignore ;)
float scale=max(pixelSize.x,pixelSize.y); //try to have a square viewport
vec3 scl=vec3(scale,-scale,0.0);

vec3 get(vec2 pt){//encoding/decoding floats in the range 0-1
	if(backbufferCounter==1)return vec3(0.0,0.0,0.0);//clear the buffer
	return texture2D(backbuffer,pt).rgb-vec3(0.5,0.5,0.0);
}
vec3 put(vec3 v){return clamp(v+vec3(0.5,0.5,0.0),0.0,1.0);}

float DE(vec2 z0){//mandelbrot distance estimate
	z0-=vec2(0.65,0.5);
	z0*=4.0;
	float dr=4.0,r=length(z0),p=2.0;
	vec2 z=z0;
	for(int i=0;i<32 && r<2.0;i++){
		float a=atan(z.y,z.x)*p;
		dr=dr*r*p+1.0;
		z=r*r*vec2(cos(a),sin(a));
		z+=z0;
		r=length(z);
	}
	return 0.5*log(r)*r/dr;
}

const vec2 ve=vec2(0.0002,0.0);
vec2 normal(vec2 z){//2d normal 
	return normalize(vec2(-DE(z-ve)+DE(z+ve),-DE(z-ve.yx)+DE(z+ve.yx)));
}

vec3 color(vec2 z) {
	vec2 pt=z*0.5+vec2(0.5);//convert to texture coords 0-1
	
	float d=DE(pt);
	if(d<0.001)return put(vec3(0.0));//we are inside the brot so return zero

	vec3 p=get(pt);//the velocity (p.x,p.y) and dye density (p.z)
	
	//create new force if near forcePt
	vec3 nf=((distance(pt,forcePt)>0.03)?vec3(0.0):vec3(forceVel,(0.03-distance(pt,forcePt))*15.15));
	
	//find advect point (where we came from)
	vec2 sp=pt-forceMult*scale*get(pt).xy;

	//meet the neighbors - these are all the points we interact with in a 0-4,0-4 grid
	vec3 p20=get(sp+2.0*scl.zy);
	vec3 p11=get(sp+scl.yy);
	vec3 p21=get(sp+scl.zy);
	vec3 p31=get(sp+scl.xy);
	vec3 p02=get(sp+2.0*scl.yz);
	vec3 p12=get(sp+scl.yz);
	vec3 p22=get(sp);
	vec3 p32=get(sp+scl.xz);
	vec3 p42=get(sp+2.0*scl.xz);
	vec3 p13=get(sp+scl.yx);
	vec3 p23=get(sp+scl.zx);
	vec3 p33=get(sp+scl.xx);
	vec3 p24=get(sp+2.0*scl.zx);
	vec3 p00=get(sp+2.0*scl.yy);
	vec3 p40=get(sp+2.0*scl.xy);
	vec3 p04=get(sp+2.0*scl.yx);
	vec3 p44=get(sp+2.0*scl.xx);
	vec3 pT=get(sp+4.0*scl.zy);
	vec3 pB=get(sp+4.0*scl.zx);
	vec3 pR=get(sp+4.0*scl.xz);
	vec3 pL=get(sp+4.0*scl.yz);
	
	//diffuse the neighborhood - about 4+ jacobies
	vec3 acc=0.5*p22+0.0625*(p21+p12+p23+p32)+0.03125*(p11+p31+p13+p33)
		+0.015625*(p20+p02+p42+p24)+0.0078125*(p00+p04+p40+p44)+0.0078125*(pT+pB+pR+pL);

	//smooth these outliers - they are used in the "second iteration" of gradient smoothing
	pT.y=pT.y*0.5+0.25*(p40.y+p00.y);
	pB.y=pB.y*0.5+0.25*(p04.y+p44.y);
	pR.x=pR.x*0.5+0.25*(p40.x+p44.x);
	pL.x=pL.x*0.5+0.25*(p00.x+p04.x);
	
	//find the gradient of the sum of gradients in 4 directions with 2 "frequencies" (kind of like the FFT version, kind of NOT)
	vec3 dlt=0.8*vec3((p42.x-p22.x+p33.y-p31.y)-(p22.x-p02.x+p13.y-p11.y),
		(p33.x-p13.x+p24.y-p22.y)-(p31.x-p11.x+p22.y-p20.y),0.0)+
		0.2*vec3((pR.x-p22.x+p44.y-p40.y)-(p22.x-pL.x+p04.y-p00.y),
		(p44.x-p04.x+pB.y-p22.y)-(p40.x-p00.x+p22.y-pT.y),0.0);
	
	//scale the gradient thingy (should be 0.25*dlt but that isn't very interesting)
	vec3 gdlt=min((projMin+length(acc)*proj),projMax)*dlt;//this is a hack for overcompensating but not exploding 
	
	//add the ingredients: new force, diffuse, gradient of summed gradients
	p=fade*(nf+acc+gdlt);
	
	if(d<0.005){//if close to the brot reflect away - normals don't seem to be working???
		vec2 nor=normal(sp);
		if(dot(p.xy,nor)<0.0)p.xy=reflect(p.xy,nor);
		p*=smoulder;//make the brot smoke on its own after heating :)
	}
	
	return put(p);
}





